from typing import List, Dict, Tuple

from krakentools.containers import SerpentMaterial
from krakentools.serpent import read_restart_file


# --- Read restart file from first part
#     Top list is over different burnup points,
#     Lower list is over different materials at that burnup point.

list_of_lists_of_materials = read_restart_file("./first.inp.wrk")

# --- Loop over materials to write something and collect the data

material_dict: Dict[Tuple[float, str], SerpentMaterial] = {}
material_names: List[str] = []

for list_of_materials in list_of_lists_of_materials:

    glob_bu = list_of_materials[0].current_global_burnup
    glob_days = list_of_materials[0].current_global_days

    print(f"\nMaterial burnups at {glob_bu} MWd/kgU.")

    for material in list_of_materials:
        print(f"Material {material.name:8}, material burnup {material.burnup:.2f}.")

        # --- Store material to dictionary for easier access later

        material_dict[(glob_bu, material.name)] = material

        # --- Store list of material names

        if material.name not in material_names:
            material_names.append(material.name)

# --- Switch some materials around:
#     Put fresh fuel to left hand side, move left hand side to right hand side

from_to_idx_list = [0, # <- goes to zone 1
                    0, # <- goes to zone 2
                    0, # <- goes to zone 3
                    1, # <- goes to zone 4
                    2, # <- goes to zone 5
                    3, # <- goes to zone 6
                    0, # <- goes to zone 7
                    0, # <- goes to zone 8
                    0, # <- goes to zone 9
                    7, # <- goes to zone 10
                    8, # <- goes to zone 11
                    9, # <- goes to zone 12
                    0, # <- goes to zone 13
                    0, # <- goes to zone 14
                    0, # <- goes to zone 15
                    13, # <- goes to zone 16
                    14, # <- goes to zone 17
                    15, # <- goes to zone 18
]

# --- Let's append our modified materials into a new list
#     (global burnup 40.0001 [which it isn't], last point in file)

glob_bu = glob_bu + 0.0001
glob_days = glob_days + 0.0001

modified_material_list = []

for name in material_names:

    # --- Try splitting the name with the z in fuel<z>idx

    parts = name.split("z")

    if len(parts) == 1:
        # This is parent material "fuel", just append as it was at the end of calculation

        my_material = material_dict[(40.0, name)]

        # --- Switch global bu

        my_material.current_global_burnup = glob_bu
        my_material.current_global_days = glob_days

        # --- The other information of the parent material will be wrong (too)

        modified_material_list.append(my_material)

        continue

    # --- This is some depletion zone

    zone_idx = int(parts[1])

    # --- Read "from index" (where to reload from) out of list

    from_idx = from_to_idx_list[zone_idx-1]

    # --- Get fresh or spent fuel

    if from_idx == 0:
        # --- If from idx is zero, should load fresh fuel (can load it from this position)

        my_material = material_dict[(0.0, name)]
    else:
        # --- From idx is not zero, get material with that name

        from_name = "fuelz"+str(from_idx)

        my_material = material_dict[(40.0, from_name)]

    # --- Set global burnup and days

    my_material.current_global_burnup = glob_bu
    my_material.current_global_days = glob_days

    # --- Set name (need to switch it if we reloaded something else)

    my_material.name = name

    # --- Put to list

    modified_material_list.append(my_material)

# --- Append our new list to others

list_of_lists_of_materials.append(modified_material_list)

# --- Print out information now and store everything to file

with open("modified.wrk","wb") as fout:
    for list_of_materials in list_of_lists_of_materials:

        glob_bu = list_of_materials[0].current_global_burnup
        glob_days = list_of_materials[0].current_global_days

        print(f"\nMaterial burnups at {glob_bu} MWd/kgU.")

        for material in list_of_materials:
            print(f"Material {material.name:8}, material burnup {material.burnup:.2f}.")

            # --- Write material to file

            material.append_to_file(fout)